# -*- coding: utf-8 -*-

'''
Module
    file_check.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class FileCheck with attribute(s) and method(s).
    Creates an API for checking operations with files.
'''

import sys
from typing import List, Optional
from os.path import splitext, isfile

try:
    from ats_utilities.checker import ATSChecker
    from ats_utilities.console_io.error import error_message
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class FileCheck(ATSChecker):
    '''
        Defines class FileCheck with attribute(s) and method(s).
        Creates an API for checking operations with files.
        Mechanism for checking files.

        It defines:

            :attributes:
                | MODES - Mode file operations.
                | TRUSTED_EXTENSIONS - List of trusted file extensions.
                | _verbose - Enable/Disable verbose option.
                | _file_path_ok - File exist, path ok.
                | _file_mode_ok - Supported file mode.
                | _file_format_ok - File format is (not) expected.
            :methods:
                | __init__ - Initials FileCheck constructor.
                | check_path - Checks file path.
                | check_mode -  Checks operation mode for file.
                | check_format - Checks file format by extension.
                | is_file_ok - Returns status for file.
    '''

    MODES: list[str] = ['r', 'w', 'a', 'b', 'x', 't', '+']
    TRUSTED_EXTENSIONS: list[str] = ['makefile']

    def __init__(self, verbose: bool = False) -> None:
        '''
            Initials FileCheck constructor.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: None
        '''
        super().__init__()
        self._verbose: bool = verbose
        self._file_path_ok: bool = False
        self._file_mode_ok: bool = False
        self._file_format_ok: bool = False
        verbose_message(self._verbose, ['init ATS check file'])

    def check_path(
        self, file_path: Optional[str], verbose: bool = False
    ) -> None:
        '''
            Checks file path.

            :param file_path: File path | None
            :type file_path: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: ATSTypeError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('str:file_path', file_path)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        file_path = str(file_path)
        if not isfile(file_path):
            self._file_path_ok = False
            error_message([f'check file {file_path}'])
        else:
            self._file_path_ok = True
            verbose_message(
                self._verbose or verbose, [f'check file path {file_path}']
            )

    def check_mode(
        self, file_mode: Optional[str], verbose: bool = False
    ) -> None:
        '''
            Checks operation mode for file.

            :param file_mode: File mode ('r', 'w', 'a', 'b', 'x', 't', '+')
            :type file_mode: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: ATSTypeError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([('str:file_mode', file_mode)])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        file_mode = str(file_mode)
        mode_checks: list[bool] = []
        for mode in list(file_mode):
            if mode not in self.MODES:
                mode_checks.append(False)
            else:
                mode_checks.append(True)
        if all(mode_checks):
            self._file_mode_ok = True
            verbose_message(
                self._verbose or verbose,
                [f'supported file mode [{file_mode}]']
            )
        else:
            self._file_mode_ok = False
            error_message([f'not supported file mode [{file_mode}]'])

    def check_format(
        self,
        file_path: Optional[str],
        file_format: Optional[str],
        verbose: bool = False
    ) -> None:
        '''
            Checks file format by extension.

            :param file_path: File path | None
            :type file_path: <Optional[str]>
            :param file_format: File format (file extension) | None
            :type file_format: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: ATSTypeError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('str:file_path', file_path), ('str:file_format', file_format)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        extension: Optional[str] = None
        file_format, file_path = str(file_format), str(file_path)
        if file_format not in self.TRUSTED_EXTENSIONS:
            extension = splitext(file_path)[1]
            extension = extension.replace('.', '')
            if extension == '':
                extension = file_format
        else:
            if file_format.capitalize() in file_path:
                extension = 'makefile'
        if not extension == file_format:
            error_message([f'check extension [{file_format}] {file_path}'])
            self._file_format_ok = False
        else:
            self._file_format_ok = True
        verbose_message(
            self._verbose or verbose, [f'checked file format {file_path}']
        )

    def is_file_ok(self) -> bool:
        '''
            Returns status for file.

            :return: True (file validated and ok) | False
            :rtype: <bool>
            :exceptions: None
        '''
        return all([
            self._file_path_ok, self._file_mode_ok, self._file_format_ok
        ])



# -*- coding: UTF-8 -*-

'''
Module
    ats_version.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class ATSVersion with attribute(s) and method(s).
    Creates an API for the ATS version in one property object.
'''

import sys
from typing import List, Optional

try:
    from ats_utilities.checker import ATSChecker
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class ATSVersion(ATSChecker):
    '''
        Defines class ATSVersion with attribute(s) and method(s).
        Creates an API for the ATS version in one property object.
        The ATS version container.

        It defines:

            :attributes:
                | _verbose - Enable/Disable verbose option.
                | _version - The ATS version.
            :methods:
                | __init__ - Initials ATSVersion constructor.
                | version - Property methods for set/get operations.
                | is_version_not_none - Checks is ATS version not None.
    '''

    def __init__(self, verbose: bool = False) -> None:
        '''
            Initials ATSVersion constructor.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: None
        '''
        super().__init__()
        self._verbose: bool = verbose
        self._version: Optional[str] = None

    @property
    def version(self) -> Optional[str]:
        '''
            Property method for getting ATS version.

            :return: The ATS version | None
            :rtype: <Optional[str]>
            :exceptions: None
        '''
        return self._version

    @version.setter
    def version(self, version: Optional[str]) -> None:
        '''
            Property method for setting ATS version.

            :param version: The ATS version | None
            :type version: <Optional[str]>
            :exceptions: ATSTypeError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([('str:version', version)])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        self._version = version
        verbose_message(self._verbose, [f'version {version}'])

    def is_version_not_none(self) -> bool:
        '''
            Checks is ATS version not None.

            :return: True (ATS version is not None) | False
            :rtype: <bool>
            :exceptions: None
        '''
        return self._version is not None



# -*- coding: UTF-8 -*-

'''
Module
    __init__.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines ats_utilities package.
'''



# -*- coding: UTF-8 -*-

'''
Module
    __init__.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class Splash with attribute(s) and method(s).
    Loads a splash screen info and adds hyperlinks.
'''

import sys
from typing import Any, Dict, Tuple, List, Optional
from time import sleep

try:
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.splash.progress_bar import ProgressBar
    from ats_utilities.splash.terminal_properties import TerminalProperties
    from ats_utilities.splash.splash_property import SplashProperty
    from ats_utilities.splash.github_infrastructure import GitHubInfrastructure
    from ats_utilities.splash.ext_infrastructure import ExtInfrastructure
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
    from ats_utilities.exceptions.ats_value_error import ATSValueError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class Splash(SplashProperty):
    '''
        Defines class Splash with attribute(s) and method(s).
        Loads a splash screen info and adds hyperlinks.
        API splash component.

        It defines:

            :attributes:
                | _verbose - Enable/Disable verbose option.
            :methods:
                | __init__ - Initials Splash constructor.
                | center - Center console line.
    '''

    def __init__(self, prop: Dict[Any, Any], verbose: bool = False) -> None:
        '''
            Initials Splash constructor.

            :param prop: Splash property in dict form
            :type prop: <: Dict[Any, Any]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: None
        '''
        super().__init__(prop, verbose)
        self._verbose: bool = verbose
        if self.validate(self._verbose):
            terminal: TerminalProperties = TerminalProperties(self._verbose)
            size: Tuple[Any, ...] = terminal.size(self._verbose)
            if bool(prop['ats_use_github_infrastructure']):
                print("\n")
                with open(prop['ats_logo_path'], 'r', encoding='utf-8') as scr:
                    for line in scr:
                        processed_line: str = line.rstrip()
                        if bool(processed_line):
                            self.center(int(size[1]), 0, processed_line)
                infrastruct = GitHubInfrastructure(prop, self._verbose)
                self.center(int(size[1]), 2, infrastruct.get_info_text())
                self.center(int(size[1]), 2, infrastruct.get_issue_text())
                self.center(int(size[1]), 2, infrastruct.get_author_text())
                print("\n")
            else:
                infrastruct = ExtInfrastructure(prop, self._verbose)
                self.center(int(size[1]), 2, infrastruct.get_info_text())
                self.center(int(size[1]), 2, infrastruct.get_issue_text())
                self.center(int(size[1]), 2, infrastruct.get_author_text())
                print("\n")
            pb: ProgressBar = ProgressBar(int(size[1]) - int(int(size[1]) / 2))
            for i in range(0, int(size[1]) - int(int(size[1]) / 2)):
                pb.set_and_plot(i + 1, int(size[1]))
                sleep(0.01)
            del pb

    def center(
        self,
        columns: int,
        additional_shifter: int,
        text: Optional[str],
        verbose: bool = False
    ) -> None:
        '''
            Center console line.

            :param columns: Colums for console session
            :type columns: <int>
            :param additional_shifter: Additional shifters
            :type additional_shifter: <int>
            :param text: Text for console session | None
            :type text: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: ATSTypeError | ATSValueError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('int:columns', columns),
            ('int:additional_shifter', additional_shifter),
            ('str:text', text)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        if not bool(text):
            raise ATSValueError('missing text')
        verbose_message(
            self._verbose or verbose,
            [f'{columns} {additional_shifter} {text}']
        )
        start_position: float = (columns / 2) - 30
        number_of_tabs = int((start_position / 8) - 1 + additional_shifter)
        print('{0}{1}'.format('\011' * number_of_tabs, text))



# -*- coding: UTF-8 -*-

'''
Module
    terminal_properties.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class TerminalProperties with attribute(s) and method(s).
    Creates an API for getting terminal properties.
'''

import sys
import os
from typing import Any, List, Tuple, Optional
from fcntl import ioctl
from termios import TIOCGWINSZ
from struct import unpack, pack

try:
    from ats_utilities.checker import ATSChecker
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
    from ats_utilities.exceptions.ats_value_error import ATSValueError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class TerminalProperties(ATSChecker):
    '''
        Defines class TerminalProperties with attribute(s) and method(s).
        Creates an API for getting terminal properties.
        API for terminal properties.

        It defines:

            :attributes:
                | _verbose - Enable/Disable verbose option.
                | _window_size - Terminal window size.
            :methods:
                | __init__ - Initials TerminalProperties constructor.
                | ioctl_get_window_size - Gets size for descriptor.
                | ioctl_for_all_descriptors - Gets size for all descriptors.
                | size - Gets size of terminal window.
    '''

    def __init__(self, verbose: bool = False) -> None:
        '''
            Initials TerminalProperties constructor.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: None
        '''
        super().__init__()
        self._verbose: bool = verbose
        self._window_size: Tuple[Any, ...]
        verbose_message(self._verbose, ['init terminal properties'])

    def ioctl_get_window_size(
        self, file_descriptor: int, verbose: bool = False
    ) -> Tuple[Any, ...]:
        '''
            Gets size for descriptor.

            :param file_descriptor: file descriptor.
            :type file_descriptor: <int>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: Window size of terminal.
            :rtype: <Tuple[Any, ...]>
            :exceptions: ATSTypeError | ATSValueError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('int:file_descriptor', file_descriptor)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        if file_descriptor < 0:
            raise ATSValueError(error_msg)
        self._window_size = unpack(
            'HHHH', ioctl(
                file_descriptor, TIOCGWINSZ, pack('HHHH', 0, 0, 0, 0)
            )
        )
        verbose_message(
            self._verbose or verbose,
            [f'terminal window size {self._window_size}']
        )
        return self._window_size

    def ioctl_for_all_descriptors(self, verbose: bool = False) -> None:
        '''
            Gets size for all descriptors.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: True (window size checked) else false
            :rtype: <bool>
            :exceptions: None
        '''
        std_in: Tuple[Any, ...] = self.ioctl_get_window_size(0)
        std_out: Tuple[Any, ...] = self.ioctl_get_window_size(1)
        std_err: Tuple[Any, ...] = self.ioctl_get_window_size(2)
        self._window_size = std_in or std_out or std_err
        verbose_message(
            self._verbose or verbose,
            [f'terminal window size {self._window_size}']
        )

    def size(self, verbose: bool = False) -> Tuple[Any, ...]:
        '''
            Gets size of terminal window.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: Window size
            :rtype: <Tuple[Any, ...]>
            :exceptions: None
        '''
        self.ioctl_for_all_descriptors()
        file_descriptor: int = os.open(os.ctermid(), os.O_RDONLY)
        self._window_size = self.ioctl_get_window_size(file_descriptor)
        os.close(file_descriptor)
        verbose_message(
            self._verbose or verbose,
            [f'terminal window size {self._window_size}']
        )
        return self._window_size



# -*- coding: UTF-8 -*-

'''
Module
    __init__.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class ATSChecker with attribute(s) and method(s).
    Creates an API for checking parameters for methods and functions.
'''

from inspect import stack
from typing import Any, List, Tuple, OrderedDict, Optional, TypeAlias
from collections import OrderedDict as OrderedDictionary

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'

CheckParams: TypeAlias = Tuple[Optional[str], Optional[int]]
ParamDesc: TypeAlias = List[Tuple[str, Any]]


class ATSChecker:
    '''
        Defines class ATSChecker with attribute(s) and method(s).
        Creates an API for checking parameters for methods and functions.
        Mechanism for checking function or method parameters (type or format).

        It defines:

            :attributes:
                | NO_ERROR - Marks no param error, error id (0).
                | TYPE_ERROR - Marks type param error, error id (1).
                | FORMAT_ERROR - Marks wrong format error, error id (2).
                | _start_message - Start segment of usage message.
                | _list_of_params - List of params for a method or function.
                | _error_type - List of mapped errors.
                | _error_type_index - Error type index.
            :methods:
                | __init__ - Initials ATSChecker constructor.
                | collect_params - Collects all params in one list.
                | usage_message - Prepares usage for method or function.
                | check_types - Checks params (types) for method or function.
                | priority_error - Sets priority error id (TYPE_ERROR).
                | check_params - Checks params for method or function.
    '''

    NO_ERROR: int = 0
    TYPE_ERROR: int = 1
    FORMAT_ERROR: int = 2

    def __init__(self) -> None:
        '''
            Initials ATSChecker constructor.

            :exceptions: None
        '''
        self._start_message: Optional[str] = None
        self._list_of_params: List[str] = []
        self._error_type: List[int] = [0, 0]
        self._error_type_index: List[int] = []

    def collect_params(self, params_desc: OrderedDict[str, Any]) -> bool:
        '''
            Collects all params in one list.

            :param params_desc: Description for params
            :type params_desc: <OrderedDict[str, Any]>
            :return: True (are collected) | False (failed to collect)
            :rtype: <bool>
            :exceptions: None
        '''
        if any([not params_desc]):
            self._error_type[1] = self.FORMAT_ERROR
            return False
        for exp_type, inst in params_desc.items():
            pname: str = exp_type.split(sep=':')[1]
            ptype: str = exp_type.split(sep=':')[0]
            self._list_of_params.append(
                f'\n    expected {pname} <{ptype}> object at {hex(id(inst))}'
            )
        return True

    def usage_message(self) -> Optional[str]:
        '''
            Prepares usage for method or function.

            :return: Usage message for method or function | None
            :rtype: <Optional[str]>
            :exceptions: None
        '''
        message: Optional[str] = self._start_message
        if bool(self._list_of_params):
            for index, param in enumerate(self._list_of_params):
                message = f'{message} {param}'
                if bool(self._error_type_index):
                    if index in set(self._error_type_index):
                        message = f'{message} wrong type'
        return message

    def check_types(self, params_desc: OrderedDict[str, Any]) -> bool:
        '''
            Checks params (types) for method or function.

            :param params_desc: Description for params
            :type params_desc: <OrderedDict[str, Any]>
            :return: True (type(s) is(are) ok) | False (type(s) is(are) not ok)
            :rtype: <bool>
            :exceptions: None
        '''
        if any([not params_desc]):
            self._error_type[1] = self.FORMAT_ERROR
            return False
        for index, (exp_type, inst) in enumerate(params_desc.items()):
            param_type_name: List[str] = exp_type.split(sep=':')
            if len(param_type_name) == 2:
                if type(inst).__name__ != param_type_name[0]:
                    self._error_type[0] = self.TYPE_ERROR
                    self._error_type_index.append(index)
                    return False
            else:
                self._error_type[1] = self.FORMAT_ERROR
                return False
        return True

    def priority_error(self) -> Optional[int]:
        '''
            Sets priority error id (TYPE_ERROR).

            :return: Priority error id (0 | 1 | 2) | None
            :rtype: <Optional[int]>
            :exceptions: None
        '''
        priority_error_id: Optional[int] = None
        if self._error_type[1] == self.FORMAT_ERROR:
            return self.FORMAT_ERROR
        if self._error_type[0] == self.TYPE_ERROR:
            priority_error_id = self.TYPE_ERROR
        if all(error_type == 0 for error_type in self._error_type):
            priority_error_id = self.NO_ERROR
        return priority_error_id

    def check_params(self, params_desc: ParamDesc) -> CheckParams:
        '''
            Checks params for method or function.

            :param params_desc: Description for params
            :type params_desc: <ParamDesc>
            :return: error message, error id (0 | 1 | 2)
            :rtype: <CheckParams>
            :exceptions: None
        '''
        func: str = stack()[1][3]
        module: str = stack()[1][1]
        self._start_message = f'\nmod: {module}\n  def: {func}()'
        fail_any_check: bool = any([
            not self.collect_params(OrderedDictionary(params_desc)),
            not self.check_types(OrderedDictionary(params_desc))
        ])
        message: Optional[str] = self.usage_message()
        error_id: Optional[int] = self.priority_error()
        if any([error_id != 0, fail_any_check]):
            message = f'{message} format wrong during checking params'
        return message, error_id



# -*- coding: UTF-8 -*-

'''
Module
    success.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class ATSSuccess with attribute(s) and method(s).
    Creates a success message container for the console log mechanism.
'''

import sys
from typing import Any, List, Optional

try:
    from colorama import init, Fore
    from ats_utilities.checker import ATSChecker
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class ATSSuccess:
    '''
        Defines class ATSSuccess with attribute(s) and method(s).
        Creates a success message container for the console log mechanism.
        Mechanism for logging success messages.

        It defines:

            :attributes:
                | _message - Success message container.
            :methods:
                | __init__ - Initials ATSSuccess constructor.
                | message - Property methods for set/get operations.
                | is_not_none - Checks is message not None.
    '''

    def __init__(self) -> None:
        '''
            Initials ATSSuccess constructor.

            :exceptions: None
        '''
        self._message: Optional[str] = None

    @property
    def message(self) -> Optional[str]:
        '''
            Property method for getting message.

            :return: Formatted success message | None
            :rtype: <Optional[str]>
            :exceptions: None
        '''
        return self._message

    @message.setter
    def message(self, message: Optional[str]) -> None:
        '''
            Property method for setting message.

            :param message: Succcess message | None
            :type message: <Optional[str]>
            :exceptions: None
        '''
        if message:
            init(autoreset=False)
            self._message = f'{Fore.GREEN}{message}{Fore.RESET}'

    def is_not_none(self) -> bool:
        '''
            Checks is message not None.

            :return: True (message is not None) | False
            :rtype: <bool>
            :exceptions: None
        '''
        return self._message is not None


def success_message(message: List[Any]) -> None:
    '''
        Shows success message.

        :param message: Message combined as list of any elements
        :type message: <List[Any]>
        :exceptions: ATSTypeError
    '''
    checker: ATSChecker = ATSChecker()
    error_msg: Optional[str] = None
    error_id: Optional[int] = None
    error_msg, error_id = checker.check_params([('list:message', message)])
    if error_id == checker.TYPE_ERROR:
        raise ATSTypeError(error_msg)
    success: ATSSuccess = ATSSuccess()
    success.message = ' '.join([str(item) for item in message])
    if success.is_not_none():
        print(f'{success.message}')



# -*- coding: utf-8 -*-

'''
Module
    __init__.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class ATSInfo with attribute(s) and method(s).
    Creates an API for the ATS information in one container object.
'''

import sys
from typing import Any, List, Dict, Optional
from datetime import datetime

try:
    from ats_utilities.info.ats_name import ATSName
    from ats_utilities.info.ats_info_ok import ATSInfoOk
    from ats_utilities.info.ats_version import ATSVersion
    from ats_utilities.info.ats_licence import ATSLicence
    from ats_utilities.info.ats_build_date import ATSBuildDate
    from ats_utilities.console_io.error import error_message
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class ATSInfo(ATSName, ATSVersion, ATSLicence, ATSBuildDate, ATSInfoOk):
    '''
        Defines class ATSInfo with attribute(s) and method(s).
        Creates an API for the ATS information in one container object.
        The ATS information container.

        It defines:

            :attributes:
                | ATS_NAME - The ATS name key.
                | ATS_VERSION - The ATS version key.
                | ATS_LICENCE - The ATS licence key.
                | ATS_BUILD_DATE - The ATS build date key.
                | ATS_BASE_INFO - The ATS base information dict.
                | _verbose - Enable/Disable verbose option.
            :methods:
                | __init__ - Initials ATSInfo constructor.
                | show_base_info - Shows ATS information.
                | is_correct - Checks information structure.
    '''

    ATS_NAME: str = 'ats_name'
    ATS_VERSION: str = 'ats_version'
    ATS_LICENCE: str = 'ats_licence'
    ATS_BUILD_DATE: str = 'ats_build_date'
    ATS_BASE_INFO: dict[int, str] = {
        1: ATS_NAME,
        2: ATS_VERSION,
        3: ATS_LICENCE,
        4: ATS_BUILD_DATE,
    }

    def __init__(self, info: Dict[Any, Any], verbose: bool = False) -> None:
        '''
            Initials ATSInfo constructor.

            :param info: The ATS basic information
            :type info: <Dict[Any, Any]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: None
        '''
        ATSName.__init__(self, verbose)
        ATSVersion.__init__(self, verbose)
        ATSLicence.__init__(self, verbose)
        ATSBuildDate.__init__(self, verbose)
        ATSInfoOk.__init__(self, verbose)
        self._verbose = verbose
        if self.is_correct(info, self._verbose):
            verbose_message(self._verbose, ['load ATS information'])
            self.name = str(info.get(self.ATS_NAME))
            self.version = str(info.get(self.ATS_VERSION))
            self.licence = str(info.get(self.ATS_LICENCE))
            self.build_date = str(info.get(self.ATS_BUILD_DATE))
            self.ats_info_ok = True

    def show_base_info(self) -> None:
        '''
            Shows ATS information.

            :exceptions: None
        '''
        if self.ats_info_ok:
            print(
                f'\n[{self.name}] ver {self.version} {datetime.now().date()}'
            )

    def is_correct(
        self, info: Dict[Any, Any], verbose: bool = False
    ) -> bool:
        '''
            Checks information structure.

            :param info: The ATS base information
            :type info: <Dict[Any, Any]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: True (all info params are ok) | False
            :rtype: <bool>
            :exceptions: ATSTypeError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([('dict:info', info)])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        verbose_message(
            self._verbose or verbose, [f'info structure {str(info)}']
        )
        all_state: List[bool] = []
        for info_key in info.keys():
            if info_key not in self.ATS_BASE_INFO.values():
                error_message([f'key not expected [{info_key}]'])
                all_state.append(False)
            else:
                if info[info_key] is None:
                    error_message([f'parameter [{info_key}] is None'])
                    all_state.append(False)
                else:
                    all_state.append(True)
        return all(all_state)



# -*- coding: utf-8 -*-

'''
Module
    __init__.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class ATSOptionParser with attribute(s) and method(s).
    Creates an option parser and an argument processor.
'''

import sys
from typing import Any, List, Tuple, Optional, Sequence, TypeAlias
from argparse import ArgumentParser, Namespace

try:
    from ats_utilities.checker import ATSChecker
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'

KnownArgs: TypeAlias = Tuple[Namespace, List[str]]
OptArgs: TypeAlias = Optional[Sequence[str]]


class ATSOptionParser(ATSChecker):
    '''
        Defines class ATSOptionParser with attribute(s) and method(s).
        Creates an option parser and an argument processor.
        Mechanism for application, tool, or script option parser.

        It defines:

            :attributes:
                | _verbose - Enable/Disable verbose option.
                | _opt_parser - Options parser.
            :methods:
                | __init__ - Initials ATSOptionParser constructor.
                | add_operation - Adds an option to the ATS parser.
                | parse_input_args - Processes arguments from the start.
                | parse_args - Processes arguments from the start.
    '''

    def __init__(
        self,
        version: Optional[str],
        epilog: Optional[str],
        description: Optional[str],
        verbose: bool = False
    ) -> None:
        '''
            Initials ATSOptionParser constructor.

            :param version: ATS version and build date | None
            :type version: <Optional[str]>
            :param epilog: ATS long description | None
            :type epilog: <Optional[str]>
            :param description: ATS author and license | None
            :type description: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: ATSTypeError
        '''
        super().__init__()
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('str:version', version),
            ('str:epilog', epilog),
            ('str:description', description)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        self._verbose: bool = verbose
        self._opt_parser: ArgumentParser = ArgumentParser(
            version, epilog, description=description
        )
        verbose_message(
            self._verbose, [f'{version}, {epilog}, {description}']
        )

    def add_operation(self, *args: str, **kwargs: Any) -> None:
        '''
            Adds an option to the ATS parser.

            :param args: List of flags for the ATS
            :type args: <str>
            :param kwargs: Arguments in shape of dictionary
            :type kwargs: <Any>
            :exceptions: None
        '''
        self._opt_parser.add_argument(*args, **kwargs)

    def parse_input_args(
        self, arguments: OptArgs, verbose: bool = False
    ) -> Namespace:
        '''
            Processes arguments from the start.

            :param arguments: Sequence of arguments | None
            :type arguments: <OptArgs>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: Namespace object
            :rtype: <Namespace>
            :exceptions: None
        '''
        args: Namespace = self._opt_parser.parse_args(arguments)
        verbose_message(self._verbose or verbose, [f'arguments {arguments}'])
        return args

    def parse_args(
            self, arguments: OptArgs, verbose: bool = False
    ) -> Namespace:
        '''
            Processes arguments from the start.

            :param arguments: Sequence of arguments | None
            :type arguments: <OptArgs>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: Namespace object
            :rtype: <Namespace>
            :exceptions: None
        '''
        args: KnownArgs = self._opt_parser.parse_known_args(arguments)
        verbose_message(self._verbose or verbose, [f'arguments {arguments}'])
        return args[0]



# -*- coding: UTF-8 -*-

'''
Module
    __init__.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class ATSCli with attribute(s) and method(s).
    Creates an API for checks and loads an information argument parser.
'''

import sys
from typing import Any, List, Optional, Sequence, TypeAlias, Union
from os.path import basename
from argparse import Namespace
from abc import abstractmethod

try:
    from ats_utilities.config_io.cfg import CfgBase
    from ats_utilities.config_io.ini import IniBase
    from ats_utilities.config_io.json import JsonBase
    from ats_utilities.config_io.xml import XmlBase
    from ats_utilities.config_io.yaml import YamlBase
    from ats_utilities.console_io.verbose import verbose_message
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'

# Optional string sequence type
ArgSeq: TypeAlias = Optional[Sequence[str]]

# Optional configuration type
Config = Optional[Union[CfgBase, IniBase, JsonBase, XmlBase, YamlBase]]


class ATSCli:
    '''
        Defines class ATSCli with attribute(s) and method(s).
        Creates an API for checks and loads an information argument parser.
        Command-line interface configuration.

        It defines:

            :attributes:
                | _config - CLI configuration object.
                | _operational - Status for tool | generator (default False).
                | _verbose - Enable/Disable verbose option.
            :methods:
                | __init__ - Initials ATSCli constructor.
                | _builder - Builds ATS cli configuration.
                | is_operational - Checks is tool | generator operational.
                | add_new_option - Adds a new option for the the CL interface.
                | parse_args - Parses the CLI arguments.
                | process - Processes and runs tool operations (Abstract).
    '''

    def __init__(
        self, info_file: Optional[str], verbose: bool = False
    ) -> None:
        '''
            Initials ATSCli constructor.

            :param info_file: Information file path | None
            :type info_file: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: None
        '''
        self._operational: bool = False
        self._config: Config = self._builder(info_file, verbose)
        self._verbose: bool = verbose
        verbose_message(self._verbose, ['init ATS CFG cli'])

    def _builder(
        self, info_file: Optional[str], verbose: bool = False
    ) -> Config:
        '''
            Builds ATS cli configuration.

            :param info_file: Information file path | None
            :type info_file: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: CLI configuration object | None
            :rtype: <Config>
            :exceptions: None
        '''
        cli_config: Config = None
        if not info_file:
            return cli_config
        file_format: str = basename(info_file).split('.')[1]
        match file_format:
            case 'cfg':
                cli_config = CfgBase(info_file, verbose)
            case 'ini':
                cli_config = IniBase(info_file, verbose)
            case 'json':
                cli_config = JsonBase(info_file, verbose)
            case 'xml':
                cli_config = XmlBase(info_file, verbose)
            case 'yaml':
                cli_config = YamlBase(info_file, verbose)
            case _:
                cli_config = None
        return cli_config

    def is_operational(self) -> bool:
        '''
            Checks is tool | generator operational.

            :return: True (tool | generator is operational) | False
            :rtype: <bool>
            :exceptions: None
        '''
        if self._config:
            self._operational = self._config.tool_operational
        return self._operational

    def add_new_option(self, *args: str, **kwargs: Any) -> None:
        '''
            Adds a new option for the CL interface.

            :param args: Arguments in string form
            :type args: <str>
            :param kwargs: arguments in Any form
            :type kwargs: <Any>
            :exceptions: None
        '''
        if self._config:
            self._config.option_parser.add_operation(*args, **kwargs)

    def parse_args(self, argv: ArgSeq) -> Optional[Namespace]:
        '''
            Parses the CLI arguments.

            :param argv: Sequence of arguments | None
            :type argv: <ArgSeq>
            :return: Options and arguments
            :rtype: <Optional[Namespace]>
            :exceptions: ATSTypeError
        '''
        if self._config:
            return self._config.option_parser.parse_args(argv)
        return None

    @abstractmethod
    def process(self, verbose: bool = False) -> bool:
        '''
            Processes and runs tool operations (Abstract).

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: True (successfully finished) | False
            :rtype: <bool>
            :exception: TypeError
        '''



# -*- coding: UTF-8 -*-

'''
Module
    ats_lookup_error.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class ATSLookupError with attribute(s) and method(s).
    Creates an exception for the lookup errors mechanism.
'''

from typing import List

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class ATSLookupError(LookupError):
    '''
        Defines class ATSLookupError with attribute(s) and method(s).
        Creates an exception for the lookup errors mechanism.

        It defines:

            :attributes:
                | None
            :methods:
                | None
    '''



# -*- coding: UTF-8 -*-

'''
Module
    progress_bar.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defineds class ProgressBar with attribute(s) and method(s).
    Loads a progressbar as part of splash screen.
'''

import sys
from typing import List

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class ProgressBar:
    '''
        Defineds class ProgressBar with attribute(s) and method(s).
        Loads a progressbar as part of splash screen.
        Progress bar component.

        It defines:

            :attributes:
                | DEFAULT_BAR_LENGTH - Length of progressbar.
                | DEFAULT_CHAR_ON - Loaded progress element.
                | DEFAULT_CHAR_OFF - Unloaded progress element.
                | _start - Start of level.
                | _end - End of level.
                | _bar_length - Progress length.
                | _level - Progress level.
                | _plotted - Plotted progress.
                | _level_chars - Level progress chars.
            :methods:
                | __init__ - Initials ProgressBar constructor.
                | set_level - Sets level of progress.
                | plot_progress - Plots progress.
                | set_and_plot - Sets and plots progress.
                | __del__ - Dunder del method for ProgressBar.
    '''

    DEFAULT_BAR_LENGTH: int = 60
    DEFAULT_CHAR_ON: str = '█'
    DEFAULT_CHAR_OFF: str = ' '

    def __init__(self, end: int, start: int = 0) -> None:
        '''
            Initials ProgressBar constructor.

            :exceptions: None
        '''
        self._end: int = end
        self._start: int = start
        self._bar_length: int = self.DEFAULT_BAR_LENGTH
        self.set_level(self._start)
        self._plotted: bool = False
        self._level: int = 0
        self._ratio: float = 0.0
        self._level_chars: int = 0

    def set_level(self, level: int) -> None:
        '''
            Sets level.

            :param level: Level of progress
            :type level: <int>
            :exceptions: None
        '''
        self._level = level
        if level < self._start:
            self._level = self._start
        if level > self._end:
            self._level = self._end
        self._ratio = (
            float(self._level - self._start) /
            float(self._end - self._start)
        )
        self._level_chars = int(self._ratio * self._bar_length)

    def plot_progress(self, columns: int) -> None:
        '''
            Plots progress.

            :param columns: Colums for open console session
            :type columns: <int>
            :exceptions: None
        '''
        start_position: float = (columns / 2) - (columns / 10)
        number_of_tabs: int = int((start_position/8) - 3)
        sys.stdout.write(
            "\r %s %3i%% %s%s".expandtabs(4) % (
                '\011' * number_of_tabs,
                int(self._ratio * 100.0),
                self.DEFAULT_CHAR_ON * int(self._level_chars),
                self.DEFAULT_CHAR_OFF *
                int(self._bar_length - self._level_chars)
            )
        )
        sys.stdout.flush()
        self._plotted = True

    def set_and_plot(self, level: int, columns: int) -> None:
        '''
            Sets and plots progress.

            :param level: Level of progress
            :type level: <int>
            :param columns: colums for open console session
            :type columns: <int>
            :exceptions: None
        '''
        old_chars: int = self._level_chars
        self.set_level(level)
        if (not self._plotted) or (old_chars != self._level_chars):
            self.plot_progress(columns)

    def __del__(self) -> None:
        '''
            Dunder del method for ProgressBar.

            :exceptions: None
        '''
        sys.stdout.write("\n")



# -*- coding: UTF-8 -*-

'''
Module
    __init__.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class JsonBase with attribute(s) and method(s).
    Loads the ATS configuration for the ATS.
'''

import sys
from typing import Any, List, Optional

try:
    from ats_utilities.info import ATSInfo
    from ats_utilities.checker import ATSChecker
    from ats_utilities.option import ATSOptionParser
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.config_io.json.json2object import Json2Object
    from ats_utilities.config_io.json.object2json import Object2Json
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
    from ats_utilities.exceptions.ats_value_error import ATSValueError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class JsonBase(ATSChecker):
    '''
        Defines class JsonBase with attribute(s) and method(s).
        Loads the ATS configuration for the ATS.
        JSON configuration-based API support.

        It defines:

            :attributes:
                | _verbose - Enable/Disable verbose option.
                | tool_operational - Control ATS operational functionality.
                | json2obj - In API for information.
                | obj2json - Out API for information.
                | option_parser - Option parser for ATS.
            :methods:
                | __init__ - Initials JsonBase constructor.
                | is_tool_ok - Checks is tool operational.
    '''

    def __init__(
        self, info_file: Optional[str], verbose: bool = False
    ) -> None:
        '''
            Initials JsonBase constructor.

            :param info_file: Information file path | None
            :type info_file: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: ATSTypeError | ATSValueError
        '''
        super().__init__()
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([('str:info_file', info_file)])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        if not bool(info_file):
            raise ATSValueError(error_msg)
        self._verbose: bool = verbose
        information: Optional[Any] = None
        self.tool_operational: bool = False
        self.json2obj: Optional[Json2Object] = Json2Object(
            info_file, self._verbose
        )
        self.obj2json: Optional[Object2Json] = Object2Json(
            info_file, self._verbose
        )
        if bool(self.json2obj) and bool(self.obj2json):
            information = self.json2obj.read_configuration(self._verbose)
        if bool(information):
            info: ATSInfo = ATSInfo(information, self._verbose)
            if info.ats_info_ok:
                self.option_parser: ATSOptionParser = ATSOptionParser(
                    f'{info.name} {info.build_date}',
                    info.version, info.licence, self._verbose
                )
                self.tool_operational = True
                verbose_message(self._verbose, ['loaded ATS JSON info'])

    def is_tool_ok(self) -> bool:
        '''
            Checks is tool operational.

            :return: True (tool is operational) | False
            :rtype: <bool>
            :exceptions: None
        '''
        return self.tool_operational



# -*- coding: UTF-8 -*-

'''
Module
    ats_build_date.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class ATSBuildDate with attribute(s) and method(s).
    Creates an API for the ATS build date in one property object.
'''

import sys
from typing import List, Optional

try:
    from ats_utilities.checker import ATSChecker
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class ATSBuildDate(ATSChecker):
    '''
        Defines class ATSBuildDate with attribute(s) and method(s).
        Creates an API for the ATS build date in one property object.
        The ATS build date container.

        It defines:

            :attributes:
                | _verbose - Enable/Disable verbose option.
                | _build_date - The ATS build date.
            :methods:
                | __init__ - Initials ATSBuildDate constructor.
                | build_date - Property methods for set/get operations.
                | is_build_date_not_none - Checks is ATS build date not None.
    '''

    def __init__(self, verbose: bool = False) -> None:
        '''
            Initials ATSBuildDate constructor.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: None
        '''
        super().__init__()
        self._verbose: bool = verbose
        self._build_date: Optional[str] = None

    @property
    def build_date(self) -> Optional[str]:
        '''
            Property method for getting ATS build date.

            :return: The ATS build date | None
            :rtype: <Optional[str]>
            :exceptions: None
        '''
        return self._build_date

    @build_date.setter
    def build_date(self, build_date: Optional[str]) -> None:
        '''
            Property method for setting ATS build date.

            :param build_date: The ATS build date | None
            :type build_date: <Optional[str]>
            :exceptions: ATSTypeError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('str:build_date', build_date)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        self._build_date = build_date
        verbose_message(self._verbose, [f'build date {build_date}'])

    def is_build_date_not_none(self) -> bool:
        '''
            Checks is ATS build date not None.

            :return: True (ATS build date is not None) | False.
            :rtype: <bool>
            :exceptions: None
        '''
        return self._build_date is not None



# -*- coding: UTF-8 -*-

'''
Module
    yaml2object.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class Yaml2Object with attribute(s) and method(s).
    Creates an API for reading a configuration from a YAML file.
'''

import sys
from typing import Any, Dict, List, Optional

try:
    from yaml import load, FullLoader
    from ats_utilities.checker import ATSChecker
    from ats_utilities.config_io import ConfFile
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
    from ats_utilities.exceptions.ats_value_error import ATSValueError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class Yaml2Object(ATSChecker):
    '''
        Defines class Yaml2Object with attribute(s) and method(s).
        Creates an API for reading a configuration from a YAML file.
        Conversion of YAML content to Python object.

        It defines:

            :attributes:
                | _EXT - File extension of the configuration file.
                | _verbose - Enable/Disable verbose option.
                | _file_path - Configuration file path.
            :methods:
                | __init__ - Initials Yaml2Object constructor.
                | read_configuration - Reads a configuration from a YAML file.
    '''

    _EXT: str = 'yaml'

    def __init__(
        self, config_file: Optional[str], verbose: bool = False
    ) -> None:
        '''
            Initials Yaml2Object constructor.

            :param config_file: Configuration file path | None
            :type config_file: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: ATSTypeError | ATSValueError
        '''
        super().__init__()
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('str:config_file', config_file)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        if not bool(config_file):
            raise ATSValueError(error_msg)
        self._verbose: bool = verbose
        self._file_path: str = str(config_file)
        verbose_message(self._verbose, [f'configuration file {config_file}'])

    def read_configuration(self, verbose: bool = False) -> Dict[Any, Any]:
        '''
            Reads a configuration from a YAML file.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: Python object
            :rtype: <Dict[Any, Any]>
            :exceptions: None
        '''
        config: Dict[Any, Any] = {}
        with ConfFile(self._file_path, 'r', self._EXT) as yaml:
            if bool(yaml):
                config = load(yaml, Loader=FullLoader)
        verbose_message(self._verbose or verbose, [f'configuration {config}'])
        return config



#!/usr/bin/env python
# -*- coding: UTF-8 -*-

'''
Module
    setup.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines setup for package ats_utilities.
'''

from typing import List, Optional
from os.path import abspath, dirname, join
from setuptools import setup

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'

THIS_DIR: str = abspath(dirname(__file__))
long_description: Optional[str] = None
with open(join(THIS_DIR, 'README.md'), encoding='utf-8') as readme:
    long_description = readme.read()
PROGRAMMING_LANG: str = 'Programming Language :: Python ::'
VERSIONS: List[str] = ['3.10', '3.11', '3.12']
SUPPORTED_PY_VERSIONS: List[str] = [
    f'{PROGRAMMING_LANG} {VERSION}' for VERSION in VERSIONS
]
PYP_CLASSIFIERS: List[str] = SUPPORTED_PY_VERSIONS
setup(
    name='ats_utilities',
    version='3.3.4',
    description='Python App/Tool/Script Utilities',
    author='Vladimir Roncevic',
    author_email='elektron.ronca@gmail.com',
    url='https://vroncevic.github.io/ats_utilities',
    license='GPL-3.0-or-later',
    long_description=long_description,
    long_description_content_type='text/markdown',
    keywords='util, config, log, option, xml, cfg, ini, json, yml, cli, meta',
    platforms='any',
    classifiers=PYP_CLASSIFIERS,
    packages=[
        'ats_utilities',
        'ats_utilities.checker',
        'ats_utilities.cli',
        'ats_utilities.config_io',
        'ats_utilities.config_io.cfg',
        'ats_utilities.config_io.ini',
        'ats_utilities.config_io.json',
        'ats_utilities.config_io.xml',
        'ats_utilities.config_io.yaml',
        'ats_utilities.console_io',
        'ats_utilities.exceptions',
        'ats_utilities.info',
        'ats_utilities.logging',
        'ats_utilities.option',
        'ats_utilities.pro_config',
        'ats_utilities.splash'
    ],
    install_requires=['colorama', 'bs4', 'lxml', 'PyYAML'],
    package_data={
        'ats_utilities': [
            'py.typed'
        ]
    }
)



# -*- coding: UTF-8 -*-

'''
Module
    __init__.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class ATSLogger with attribute(s) and method(s).
    Creates an API for the ATS logging mechanism.
'''

import sys
from typing import Any, List, Dict, Optional
from logging import (
    getLogger, basicConfig, Logger, DEBUG, WARNING, CRITICAL, ERROR, INFO
)

try:
    from ats_utilities.checker import ATSChecker
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
    from ats_utilities.exceptions.ats_bad_call_error import ATSBadCallError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class ATSLogger(ATSChecker):
    '''
        Defines class ATSLogger with attribute(s) and method(s).
        Creates an API for the ATS logging mechanism.
        ATS logger mechanism.

        It defines:

            :attributes:
                | LOG_MSG_FORMAT - Log message format.
                | LOG_DATE_FORMAT - Log date format.
                | ATS_DEBUG - Debug log level.
                | ATS_WARNING - Warning log level.
                | ATS_CRITICAL - Critical log level.
                | ATS_ERROR - Error log level.
                | ATS_INFO - Info log level.
                | _verbose - Enable/Disable verbose option.
            :methods:
                | __init__ - Initials ATSLogger constructor.
                | write_log - Writes message to log file.
    '''

    LOG_MSG_FORMAT: str = '%(asctime)s - %(levelname)s - %(message)s'
    LOG_DATE_FORMAT: str = '%m/%d/%Y %I:%M:%S %p'
    ATS_DEBUG: int = DEBUG
    ATS_WARNING: int = WARNING
    ATS_CRITICAL: int = CRITICAL
    ATS_ERROR: int = ERROR
    ATS_INFO: int = INFO

    def __init__(
        self,
        ats_name: Optional[str],
        ats_log_stdout: bool = True,
        ats_log_file: Optional[str] = None,
        ats_logger_status: bool = False,
        verbose: bool = False
    ) -> None:
        '''
            Initials ATSLogger constructor.

            :param ats_name: ATS name | None
            :type ats_name: <Optional[str]>
            :param ats_log_file: Log file path of ATS | None
            :type ats_log_file: <Optional[str]>
            :param ats_log_stream: Log stream | None
            :type ats_log_stream: <Optional[TextIOWrapper]>
            :param ats_logger_status: Logging status (defaault False)
            :type ats_logger_status: <bool>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: ATSTypeError | ATSBadCallError
        '''
        super().__init__()
        self.logger_name: Optional[str] = ats_name
        self.logger_path: Optional[str] = ats_log_file
        log_config: Dict[str, Any] = {
            'format': self.LOG_MSG_FORMAT,
            'datefmt': self.LOG_DATE_FORMAT,
            'level': DEBUG
        }
        if bool(ats_log_file) and not ats_log_stdout:
            log_config['filename'] = ats_log_file
        elif not bool(ats_log_file) and not ats_log_stdout:
            raise ATSTypeError(f'check log file {ats_log_file}')
        elif bool(ats_log_file) and ats_log_stdout:
            raise ATSBadCallError('file log or stream log can be supported')
        basicConfig(**log_config)
        self.logger: Logger = getLogger(self.logger_name)
        self.logger_status: bool = ats_logger_status
        self._verbose: bool = verbose
        verbose_message(self._verbose, ['init ATS logger'])

    def write_log(
        self, message: Optional[str], ctrl: int, verbose: bool = False
    ) -> bool:
        '''
            Writes message to log file.

            :param message: Log message for log file | None
            :type message: <Optional[str]>
            :param ctrl: Control flag (debug, warning, critical, errors, info)
            :type ctrl: <int>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: True (successfully logged message) | False
            :rtype: <bool>
            :exceptions: ATSTypeError | ATSBadCallError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('str:message', message), ('int:ctrl', ctrl)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        status: bool = False
        if not self.logger_status:
            # Skip logging messages
            return status
        verbose_message(self._verbose or verbose, [f'{message} {str(ctrl)}'])
        if self.logger_status:
            match ctrl:
                case self.ATS_DEBUG:
                    self.logger.debug(message)
                    status = True
                case self.ATS_WARNING:
                    self.logger.warning(message)
                    status = True
                case self.ATS_CRITICAL:
                    self.logger.critical(message)
                    status = True
                case self.ATS_ERROR:
                    self.logger.error(message)
                    status = True
                case self.ATS_INFO:
                    self.logger.info(message)
                    status = True
                case _:
                    raise ATSBadCallError(
                        f'not supported log level [{str(ctrl)}]'
                    )
        return status



# -*- coding: UTF-8 -*-

'''
Module
    __init__.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class ConfFile with attribute(s) and method(s).
    Creates an API for the configuration context manager.
'''

import sys
from typing import Any, List, Tuple, Dict, IO, Optional, TypeAlias

try:
    from ats_utilities.config_io.file_check import FileCheck
    from ats_utilities.checker import ATSChecker
    from ats_utilities.console_io.error import error_message
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
    from ats_utilities.exceptions.ats_value_error import ATSValueError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'

# Optional bytes, str IO type
File: TypeAlias = Optional[IO[str]]


class ConfFile(FileCheck):
    '''
        Defines class ConfFile with attribute(s) and method(s).
        Creates an API for the configuration context manager.
        Configuration file context manager.

        It defines:

            :attributes:
                | _verbose - Enable/Disable verbose option.
                | _file_path - Configuration file name.
                | _file_mode - File mode.
                | _file_format - File format.
                | _file - File object.
            :methods:
                | __init__ - Initials ConfFile constructor.
                | __enter__ - Opens configuration file in mode.
                | __exit__ - Closes configuration file.
    '''

    def __init__(
        self,
        file_path: Optional[str],
        file_mode: Optional[str],
        file_format: Optional[str],
        verbose: bool = False
    ) -> None:
        '''
            Initials ConfFile constructor.

            :param file_path: Configuration file path | None
            :type file_path: <Optional[str]>
            :param file_mode: File mode for configuration file | None
            :type file_mode: <Optional[str]>
            :param file_format: File format | None
            :type file_format: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: ATSTypeError | ATSValueError
        '''
        checker: ATSChecker = ATSChecker()
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = checker.check_params([
            ('str:file_path', file_path),
            ('str:file_mode', file_mode),
            ('str:file_format', file_format)
        ])
        if error_id == checker.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        if not bool(file_path):
            raise ATSValueError('mising file path')
        if not bool(file_mode):
            raise ATSValueError('missing file mode')
        if not bool(file_format):
            raise ATSValueError('missing file format')
        super().__init__(verbose)
        self._verbose: bool = verbose
        self._file: File = None
        self._file_path: Optional[str] = None
        self._file_mode: Optional[str] = None
        self._file_format: Optional[str] = None
        self.check_path(str(file_path), self._verbose)
        self.check_mode(str(file_mode), self._verbose)
        self.check_format(str(file_format), file_format, self._verbose)
        if self.is_file_ok():
            self._file_path = file_path
            self._file_mode = file_mode
            self._file_format = file_format
        verbose_message(self._verbose, [f'set file {file_path} {file_mode}'])

    def __enter__(self) -> File:
        '''
            Opens configuration file in mode.

            :return: File IO object | None
            :rtype: <File>
            :exceptions: None
        '''
        if self.is_file_ok():
            mode: str = "r" if not bool(
                str(self._file_mode)
            ) else str(self._file_mode)
            self._file = open(str(self._file_path), mode, encoding='utf-8')
        else:
            error_message([f'check file {str(self._file_path)}'])
            self._file = None
        return self._file

    def __exit__(
        self, *args: Tuple[Any, ...], **kwargs: Dict[Any, Any]
    ) -> None:
        '''
            Closes configuration file.

            :param *args: List of arguments
            :type file_path: <Tuple[Any, ...]>
            :param **kwargs: Dictionary of mapped arguments
            :type file_path: <Dict[Any, Any]>
            :exceptions: None
        '''
        if self._file is not None and not self._file.closed:
            self._file.close()



# -*- coding: UTF-8 -*-

'''
Module
    ini2object.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class Ini2Object with attribute(s) and method(s).
    Creates an API for reading configuration from an INI file.
'''

import sys
from typing import List, Optional
from configparser import ConfigParser

try:
    from ats_utilities.checker import ATSChecker
    from ats_utilities.config_io import ConfFile
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
    from ats_utilities.exceptions.ats_value_error import ATSValueError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class Ini2Object(ATSChecker):
    '''
        Defines class Ini2Object with attribute(s) and method(s).
        Creates an API for reading configuration from an INI file.
        Conversion of INI config to Python object.

        It defines:

            :attributes:
                | _EXT - File extension of the configuration file.
                | _verbose - Enable/Disable verbose option.
                | _file_path - Configuraiton file path.
            :methods:
                | __init__ - Initials Ini2Object constructor.
                | read_configuration - reads configuration from an INI file.
    '''

    _EXT: str = 'ini'

    def __init__(
        self, config_file: Optional[str], verbose: bool = False
    ) -> None:
        '''
            Initials Ini2Object constructor.

            :param config_file: Configuration file path | None
            :type config_file: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: ATSTypeError | ATSTypeError
        '''
        super().__init__()
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('str:config_file', config_file)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        if not bool(config_file):
            raise ATSValueError(error_msg)
        self._verbose: bool = verbose
        self._file_path: str = str(config_file)
        verbose_message(self._verbose, [f'configuration file {config_file}'])

    def read_configuration(
        self, verbose: bool = False
    ) -> Optional[ConfigParser]:
        '''
            Reads a configuration from an INI file.

            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :return: Configuration object | None
            :rtype: <Optional[ConfigParser]>
            :exceptions: None
        '''
        config: Optional[ConfigParser] = None
        with ConfFile(self._file_path, 'r', self._EXT) as ini:
            if bool(ini):
                config = ConfigParser()
                config.read_file(ini)
        verbose_message(self._verbose or verbose, [f'configuration {config}'])
        return config



# -*- coding: UTF-8 -*-

'''
Module
    object2ini.py
Copyright
    Copyright (C) 2017 - 2026 Vladimir Roncevic <elektron.ronca@gmail.com>
    ats_utilities is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by the
    Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
    ats_utilities is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
    See the GNU General Public License for more details.
    You should have received a copy of the GNU General Public License along
    with this program. If not, see <http://www.gnu.org/licenses/>.
Info
    Defines class Object2Ini with attribute(s) and method(s).
    Creates an API for writing configuration to an INI file.
'''

import sys
from typing import List, Optional
from configparser import ConfigParser

try:
    from ats_utilities.checker import ATSChecker
    from ats_utilities.config_io import ConfFile
    from ats_utilities.console_io.verbose import verbose_message
    from ats_utilities.exceptions.ats_type_error import ATSTypeError
    from ats_utilities.exceptions.ats_value_error import ATSValueError
except ImportError as ats_error_message:  # pragma: no cover
    # Force exit python #######################################################
    sys.exit(f'\n{__file__}\n{ats_error_message}\n')  # pragma: no cover

__author__: str = 'Vladimir Roncevic'
__copyright__: str = '(C) 2026, https://vroncevic.github.io/ats_utilities'
__credits__: List[str] = ['Vladimir Roncevic', 'Python Software Foundation']
__license__: str = 'https://github.com/vroncevic/ats_utilities/blob/dev/LICENSE'
__version__: str = '3.3.4'
__maintainer__: str = 'Vladimir Roncevic'
__email__: str = 'elektron.ronca@gmail.com'
__status__: str = 'Updated'


class Object2Ini(ATSChecker):
    '''
        Defines class Object2Ini with attribute(s) and method(s).
        Creates an API for writing configuration to an INI file.
        Conversion of Python object to INI content.

        It defines:

            :attributes:
                | _EXT - File extension of the configuration file.
                | _verbose - Enable/Disable verbose option.
                | _file_path - Confguration file path.
            :methods:
                | __init__ - Initials Object2Ini constructor.
                | write_configuration - Writes configuration to an INI file.
    '''

    _EXT: str = 'ini'

    def __init__(
        self, config_file: Optional[str], verbose: bool = False
    ) -> None:
        '''
            Initials Object2Ini constructor.

            :param config_file: Configuration file path | None
            :type config_file: <Optional[str]>
            :param verbose: Enable/Disable verbose option
            :type verbose: <bool>
            :exceptions: ATSTypeError | ATSTypeError
        '''
        super().__init__()
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('str:config_file', config_file)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        if not bool(config_file):
            raise ATSValueError(error_msg)
        self._verbose: bool = verbose
        self._file_path: str = str(config_file)
        verbose_message(self._verbose, [f'configuration file {config_file}'])

    def write_configuration(
        self, config: Optional[ConfigParser], verbose: bool = False
    ) -> bool:
        '''
            Writes configuration to a INI file.

            :param config: Configuration object | None
            :type config: <Optional[ConfigParser]>
            :param verbose: enable/disable verbose option
            :type verbose: <bool>
            :return: True (configuration written to file) | False
            :rtype: <bool>
            :exception: ATSTypeError | ATSTypeError
        '''
        error_msg: Optional[str] = None
        error_id: Optional[int] = None
        error_msg, error_id = self.check_params([
            ('ConfigParser:config', config)
        ])
        if error_id == self.TYPE_ERROR:
            raise ATSTypeError(error_msg)
        if not bool(config):
            raise ATSValueError(error_msg)
        status = False
        verbose_message(self._verbose or verbose, [f'configuration {config}'])
        if bool(config):
            if not bool(config.sections()):
                return status
            with ConfFile(self._file_path, 'w', self._EXT) as ini:
                if bool(ini):
                    config.write(ini, space_around_delimiters=True)
                    status = True
        return status
